@@ -166,6 +166,8 @@ static XtResource resources[] = {
        offset(pref_width), XtRImmediate, (XtPointer)1},
     {XtNpreferredHeight, XtCPreferredHeight, XtRDimension, sizeof(Dimension),
        offset(pref_height), XtRImmediate, (XtPointer)1},
+    {XtNsafeDir, XtCSafeDir, XtRBoolean, sizeof(Boolean),
+          offset(safeDir), XtRImmediate, (XtPointer)True },
     {XtNsafer, XtCSafer, XtRBoolean, sizeof(Boolean),
           offset(safer), XtRImmediate, (XtPointer)True },
     {XtNinterpreter, XtCInterpreter, XtRString, sizeof(String),
@@ -964,6 +966,7 @@ SetValues(Widget current, Widget request, Widget new, ArgList unused1, Cardinal
 	   (cgvw->ghostview.quiet != ngvw->ghostview.quiet)			||
 	   (cgvw->ghostview.infoVerbose != ngvw->ghostview.infoVerbose)			||
 	   (cgvw->ghostview.safer != ngvw->ghostview.safer)			||
+	   (cgvw->ghostview.safeDir != ngvw->ghostview.safeDir)			||
 	   strcmp(cfilename, rfilename)						||
            (cgvw->ghostview.orientation != ngvw->ghostview.orientation)		||
 	   (cgvw->ghostview.use_bpixmap != ngvw->ghostview.use_bpixmap)		||  
@@ -1421,6 +1424,7 @@ StartInterpreter(w)
 	 if (*dptr) *dptr++ = '\0';
 	 if (argc + 2 >= NUM_ARGS) {
 	    fprintf(stderr, "Too many arguments to interpreter.\n");
+	    clean_safe_tempdir();
 	    exit(EXIT_STATUS_ERROR);
 	 }
 	 while (isspace(*dptr)) dptr++;
@@ -1444,6 +1448,7 @@ StartInterpreter(w)
     if (gvw->ghostview.quiet) argv[argc++] = "-dQUIET";
     if (gvw->ghostview.safer) 
       {
+	argv[argc++] = "-P-";
 	argv[argc++] = "-dSAFER";
 #   ifdef ALLOW_PDF
 	/* The file created by pdf2dsc opens the original
@@ -1462,6 +1467,7 @@ StartInterpreter(w)
 	    if (*cptr) *cptr++ = '\0';
 	    if (argc + 2 >= NUM_ARGS) {
 		fprintf(stderr, "Too many arguments to interpreter.\n");
+		clean_safe_tempdir();
 		exit(EXIT_STATUS_ERROR);
 	    }
 	    while (isspace(*cptr)) cptr++;
@@ -1504,6 +1510,7 @@ StartInterpreter(w)
 	ret = pipe(std_in);
 	if (ret == -1) {
 	    perror("Could not create pipe");
+	    clean_safe_tempdir();
 	    exit(EXIT_STATUS_ERROR);
 	}
     } else if (strcmp(gvw->ghostview.filename, "-")) {
@@ -1511,6 +1518,7 @@ StartInterpreter(w)
           ret = pipe(std_in);
           if (ret == -1) {
              perror("Could not create pipe");
+	     clean_safe_tempdir();
              exit(EXIT_STATUS_ERROR);
           }
 #      else
@@ -1520,11 +1528,13 @@ StartInterpreter(w)
     ret = pipe(std_out);
     if (ret == -1) {
 	perror("Could not create pipe");
+	clean_safe_tempdir();
 	exit(EXIT_STATUS_ERROR);
     }
     ret = pipe(std_err);
     if (ret == -1) {
 	perror("Could not create pipe");
+	clean_safe_tempdir();
 	exit(EXIT_STATUS_ERROR);
     }
 
@@ -1574,6 +1584,14 @@ StartInterpreter(w)
 	      close(std_in[0]);
 #          endif
 	}
+    	if (gvw->ghostview.safeDir) {
+		if (chdir(gv_safe_gs_workdir) != 0) {
+			sprintf(buf, "Chdir to %s failed",
+					gv_safe_gs_workdir);
+			perror(buf);
+			_exit(EXIT_STATUS_ERROR);
+		}
+    	}
 	execvp(argv[0], argv);
 	sprintf(buf, execOfFailedLabel, argv[0]);
 	perror(buf);
