@@ -40,9 +40,7 @@
 #include <stdlib.h>
 #include <math.h>
 
-#ifdef HAVE_INTTYPES_H
-  #include <inttypes.h>
-#endif
+#include <inttypes.h>
 
 #include "paths.h"
 #include INC_X11(IntrinsicP.h)
@@ -1045,6 +1043,7 @@ static void Redisplay(w, event, region)
    } else {
       /* should never come here */
       printf("  Scrollbarwidget: no event forced exit\n");
+      clean_safe_tempdir();
       exit(0);
    }
    TOPLOC = newtop;
