@@ -38,9 +38,7 @@
 #include <stdlib.h>
 #include <math.h>
 
-#ifdef HAVE_INTTYPES_H
-  #include <inttypes.h>
-#endif
+#include <inttypes.h>
 
 #include "paths.h"
 #include INC_X11(Intrinsic.h)
@@ -1177,3 +1175,14 @@ action_watchFile(w, event, params, num_params)
   ENDMESSAGE(action_watchFile)
 }
 
+
+
+void
+clean_safe_tempdir()
+{
+   if (gv_safe_gs_tempdir)
+   {
+      chdir("/");
+      rmdir(gv_safe_gs_workdir);
+   }
+}
