@@ -46,7 +46,7 @@ static String gv_nls_noint_de_dat [] = {
 "GV*mediasLabel.label:			Mediengroessen",
 "GV*optionfsPopup.title:			Dateiauswahloptionen",
 "GV*optiongvPopup*mediaLabel.label:	\\ Papiergroesse \\ ",
-"GV*convLabel.label:	?Convert PDF?",
+"GV*convLabel.label:	Konvertiere PDF",
 "GV*zoomPopup.title:			Lupe",
 "GV*optiongvPopup.title:			gv Einstellungen",
 "GV*confirmPrint.label:			Drucken bestaetigen",
@@ -488,7 +488,7 @@ static String gv_nls_de_UTF_8_dat [] = {
 "GV*mediasLabel.label:			Mediengrößen",
 "GV*optionfsPopup.title:			Dateiauswahloptionen",
 "GV*optiongvPopup*mediaLabel.label:	\\ Papiergröße \\ ",
-"GV*convLabel.label:	?Convert PDF?",
+"GV*convLabel.label:	Konvertiere PDF",
 "GV*zoomPopup.title:			Lupe",
 "GV*optiongvPopup.title:			gv Einstellungen",
 "GV*confirmPrint.label:			Drucken bestätigen",
