@@ -46,9 +46,7 @@
 #include <math.h>
 #include <getopt.h>
 
-#ifdef HAVE_INTTYPES_H
-  #include <inttypes.h>
-#endif
+#include <inttypes.h>
 
 #include "paths.h"
 #include INC_X11(Intrinsic.h)
@@ -160,6 +158,8 @@ enum
     NOANTIALIAS_ARG,
     SAFER_ARG,
     NOSAFER_ARG,
+    SAFEDIR_ARG,
+    NOSAFEDIR_ARG,
     PIXMAP_ARG,
     NOPIXMAP_ARG,
     COLOR_ARG,
@@ -209,6 +209,8 @@ static struct option const GNU_longOptions[] =
     {"noantialias", no_argument, NULL, NOANTIALIAS_ARG},
     {"safer", no_argument, NULL, SAFER_ARG},
     {"nosafer", no_argument, NULL, NOSAFER_ARG},
+    {"safedir", no_argument, NULL, SAFEDIR_ARG},
+    {"nosafedir", no_argument, NULL, NOSAFEDIR_ARG},
     {"pixmap", no_argument, NULL, PIXMAP_ARG},
     {"nopixmap", no_argument, NULL, NOPIXMAP_ARG},
     {"color", no_argument, NULL, COLOR_ARG},
@@ -330,8 +332,9 @@ int main(argc, argv)
   int  argc;
   char *argv[];
 {
-
   MAINBEGINMESSAGE(main)
+  gv_safe_gs_workdir = GV_LIBDIR "/safe-gs-workdir";
+  gv_safe_gs_tempdir = 0;
 
   {
     Arg          args[20];
@@ -371,6 +374,8 @@ int main(argc, argv)
     noantialias_p = 0;
     safer_p = 0;
     nosafer_p = 0;
+    safedir_p = 0;
+    nosafedir_p = 0;
     pixmap_p = 0;
     nopixmap_p = 0;
     color_p = 0;
@@ -559,6 +564,12 @@ int main(argc, argv)
 	 case NOSAFER_ARG:
 	   nosafer_p = 1;
 	   break;
+	 case SAFEDIR_ARG:
+	   safedir_p = 1;
+	   break;
+	 case NOSAFEDIR_ARG:
+	   nosafedir_p = 1;
+	   break;
 	 case ANTIALIAS_ARG:
 	   antialias_p = 1;
 	   break;
@@ -653,6 +664,20 @@ int main(argc, argv)
        exit(EXIT_STATUS_ERROR);
      }
 
+     if (access(gv_safe_gs_workdir, R_OK | X_OK))
+     {
+        char buffer[512];
+	strcpy(buffer, "/tmp/gv-safe-workdir-XXXXXX");
+        gv_safe_gs_workdir = mkdtemp(buffer);
+	gv_safe_gs_tempdir = 1;
+
+	if (!gv_safe_gs_workdir)
+	{
+	   perror("Cannot create safe workdir");
+	   exit(1);
+	}
+     }  
+
    /*### getting resources ############################################*/
    gv_database = resource_buildDatabase (gv_display,
                                          gv_class,
@@ -683,6 +708,7 @@ int main(argc, argv)
     XtGetApplicationResources(toplevel,(XtPointer) &app_res,resources,XtNumber(resources),NULL,ZERO);
     if (!resource_checkResources(gv_name,app_res.version,versionCompatibility)) {
       XtDestroyApplicationContext(app_con);
+      clean_safe_tempdir();
       exit(EXIT_STATUS_ERROR);
     }
 
@@ -1020,6 +1046,7 @@ int main(argc, argv)
 #if 0
        if (!layout) {
           fprintf(stderr,"  %s: Error, layout resource not found\n",versionIdentification[0]);
+	  clean_safe_tempdir();
           exit(EXIT_STATUS_ERROR);
        }
 #endif
@@ -1163,6 +1190,8 @@ int main(argc, argv)
       Boolean b;
 									n=0;
             XtSetArg(args[n], XtNinterpreter,gv_gs_interpreter);	n++;
+            b = gv_gs_safeDir ? True : False;
+            XtSetArg(args[n], XtNsafeDir,b);                            n++;
             b = gv_gs_safer ? True : False;
             XtSetArg(args[n], XtNsafer,b);                              n++;
             b = gv_gs_quiet ? True : False;
@@ -1193,6 +1222,7 @@ int main(argc, argv)
    if (gv_filename && strcmp(gv_filename, "-")) {
       if (misc_changeFile(gv_filename)) {
 	open_fail_error(errno,GV_ERROR_OPEN_FAIL,gv_filename,1);
+	clean_safe_tempdir();
 	exit(EXIT_STATUS_ERROR);
       } else {
         GV_XtFree(gv_filename_old);
@@ -1424,6 +1454,9 @@ void main_setGhostscriptResources(db)
   s = resource_getResource(db,gv_class,gv_name,"arguments","Arguments");
   if (s) gv_gs_arguments = s;
 
+  s = resource_getResource(db,gv_class,gv_name,"gsSafeDir",NULL);
+  if (!strcasecmp(s,"true"))  gv_gs_safeDir = 1; else gv_gs_safeDir = 0;
+
   s = resource_getResource(db,gv_class,gv_name,"gsSafer",NULL);
   if (s && !strcasecmp(s,"true"))  gv_gs_safer = 1; else gv_gs_safer = 0;
   s = resource_getResource(db,gv_class,gv_name,"safer","Safer");
