@@ -29,6 +29,8 @@
 **           GNU Project
 **
 */
+/* for canonicalize_file_name: */
+#define _GNU_SOURCE 1
 #include "ac_config.h"
 
 /*
@@ -68,9 +70,7 @@
 #include <ctype.h>
 
 #include "ac_config.h"
-#ifdef HAVE_INTTYPES_H
-  #include <inttypes.h>
-#endif
+#include <inttypes.h>
 
 #ifndef SEEK_SET
 #   define SEEK_SET 0
@@ -420,14 +420,9 @@ String misc_changeFile(name)
     if (!b && file_fileIsNotUseful(p)) sprintf(p,"%s.ps",name);  else b = True;
     if (!b && file_fileIsNotUseful(p)) sprintf(p,"%s.pdf",name); else b = True;
     if (!b)                            strcpy(p,name);
-    dir=file_getDirOfPath(p);
-    if(strcmp(dir,p)) {
-      chdir(dir);
-      /* Strip off directory from p to satisfy GS 8.00 security change */
-      q=p;
-      while(*dir && *dir == *q) {dir++; q++;}
-      strcpy(p,q);
-    }
+    name = canonicalize_file_name(p);
+    if (name)
+	    p = name;
   }
   name = p;
   INFSMESSAGE(trying to open,name)
@@ -910,12 +905,39 @@ setup_ghostview()
 
     INFMESSAGE(scanning file for structure information)
     gv_filename_dsc = gv_filename_unc = NULL;
-    doc_scanFile( &gv_psfile,&doc,
+    {
+       char* tmp = GV_malloc(1512	);
+       char* src = gv_gs_cmd_scan_pdf;
+       char* dest = tmp;
+       int spaceFound = 0;
+       
+       if (strstr(gv_gs_cmd_scan_pdf, "-P") || !gv_gs_safer)
+          strcpy(tmp, gv_gs_cmd_scan_pdf);
+       else
+       {
+          while (*src)
+          {
+             int isSpace = *src == ' ';
+             *(dest++) = *(src++);
+	     if (!spaceFound && isSpace)
+	     {
+	        strcpy(dest, "-P- -dSAFER -dDELAYSAFER ");
+	        dest+=25;
+		spaceFound = 1;
+	     }
+          }
+	  *dest = 0;
+       }
+
+       doc_scanFile( &gv_psfile,&doc,
 		  gv_filename,
 		  gv_filename_raw,
-		  &gv_filename_dsc,gv_gs_cmd_scan_pdf,
+		  &gv_filename_dsc,tmp,
 		  &gv_filename_unc,gv_uncompress_command,
-		  gv_scanstyle);
+		  gv_scanstyle, gv_gs_safeDir);
+
+       GV_free(tmp);
+    }
     {
       int m;
       m = gv_pagemedia;
