@@ -100,8 +100,10 @@
 #include "d_memdebug.h"
 extern Media *gv_medias;
 extern String gv_pdf_password;
+extern String gv_safe_gs_workdir;
 extern int gv_infoSkipErrors;
 
+
 #ifdef BSD4_2
 #define memset(a,b,c) bzero(a,c)
 #endif
@@ -327,6 +329,7 @@ static char    *empty_string = "";
 static void ps_dynMemExhaust(void)
 {
    fprintf(stderr,"Fatal Error: Dynamic memory exhausted.\n");
+   clean_safe_tempdir();
    exit(EXIT_STATUS_FATAL);
 }
 #define CHECK_MALLOCED(aaa)  if (!(aaa)) ps_dynMemExhaust()
@@ -336,7 +339,7 @@ static void ps_dynMemExhaust(void)
 /*###########################################################*/
 
 struct document *
-psscan(fileP,filename,filename_raw,filename_dscP,cmd_scan_pdf,filename_uncP,cmd_uncompress,scanstyle)
+psscan(fileP,filename,filename_raw,filename_dscP,cmd_scan_pdf,filename_uncP,cmd_uncompress,scanstyle,gv_gs_safeDir)
     FILE **fileP;
     char *filename;
     char *filename_raw;
@@ -345,6 +348,7 @@ psscan(fileP,filename,filename_raw,filename_dscP,cmd_scan_pdf,filename_uncP,cmd_
     char **filename_uncP;
     char *cmd_uncompress;
     int scanstyle;
+    int gv_gs_safeDir;
 {
     FILE *file;
     struct document *doc;
@@ -410,7 +414,14 @@ psscan(fileP,filename,filename_raw,filename_dscP,cmd_scan_pdf,filename_uncP,cmd_
       char cmd[512];
       char s[512];
       mode_t old_umask;
-      filename_unc=file_getTmpFilename(NULL,filename_raw);
+      filename_unc=file_getTmpFilename(NULL, filename_raw, NULL);
+      if (!filename_unc) {
+	NotePopupShowMessage("Cannot create temporary file!");
+	ENDMESSAGE(psscan)
+        return(retval);
+      }
+      old_umask = umask(0077);
+
       quoted_filename = quote_filename(filename);
       quoted_filename_unc = quote_filename(filename_unc);
       if (memcmp(b, "BZh", 3) == 0) {
@@ -421,7 +432,6 @@ psscan(fileP,filename,filename_raw,filename_dscP,cmd_scan_pdf,filename_uncP,cmd_
       GV_XtFree(quoted_filename);
       GV_XtFree(quoted_filename_unc);
 
-      old_umask = umask(0077);
 
       INFMESSAGE(is compressed)
       INFSMESSAGE(uncompress command,cmd)
@@ -445,7 +455,7 @@ unc_ok:
       if (!tmpfile) goto unc_exec_failed;
       fclose(*fileP);
       *fileP = tmpfile;
-      retval = psscan(fileP,filename_unc,filename_raw,filename_dscP,cmd_scan_pdf,NULL,NULL,scanstyle);
+      retval = psscan(fileP,filename_unc,filename_raw,filename_dscP,cmd_scan_pdf,NULL,NULL,scanstyle, gv_gs_safeDir);
 #if 0
       if (!retval) {
 	sprintf(s,"333 Scanning\n%s\nfailed.",filename_unc);
@@ -520,7 +530,14 @@ unc_ok:
       String tmp_filename;
       int tmpfd, tmp_fd;
 
-      filename_dsc=file_getTmpFilename(NULL,filename_raw);
+      filename_dsc=file_getTmpFilename(NULL, filename_raw, NULL);
+      if (!filename_dsc) {
+	NotePopupShowMessage("Cannot create temporary file!");
+	if (tmpfile) fclose(tmpfile);
+        ps_io_exit(fd);
+	ENDMESSAGE(psscan)
+        return(retval);
+      }
       /*      sprintf(cmd,cmd_scan_pdf,filename,filename_dsc); */
       quoted_filename = quote_filename(filename);
       quoted_filename_dsc = quote_filename(filename_dsc);
@@ -559,14 +576,24 @@ unc_ok:
 
       old_umask = umask(0077);
       INFMESSAGE(is PDF)
+      if (gv_gs_safeDir) {
+        if (chdir(gv_safe_gs_workdir) != 0) {
+	  strcpy(s, "Chdir to ");
+	  strcat(s, gv_safe_gs_workdir);
+	  strcat(s, " failed");
+	  goto scan_failed;
+	}
+      }
       INFSMESSAGE(scan command,cmd)
-      
-      tmp_filename=file_getTmpFilename(NULL,filename_raw);
 
-      tmp_fd = open(tmp_filename, O_WRONLY|O_CREAT|O_TRUNC, 0666);
+      tmp_filename=file_getTmpFilename(NULL, filename_raw, &tmp_fd);
+      if (!tmp_filename) {
+	strcpy(s, "Cannot create temporary file!");
+	goto scan_failed;
+      }
       tmpfd = dup(2);
-      close(2); dup2( tmp_fd, 2);
-      
+      close(2); dup2( tmp_fd, 2); close(tmp_fd);
+
 
       if (system(cmd) || file_fileIsNotUseful(filename_dsc)) {
         char line[1000];
@@ -579,8 +606,7 @@ unc_ok:
 	close(2);
 	dup2(tmpfd,2);
 	close(tmpfd);
-	close(tmp_fd);
-	
+
         tmp_file = fopen( tmp_filename, "r" );
 	while ( fgets( line, 999, tmp_file) )
 	{
@@ -629,7 +655,7 @@ scan_ok:
       if (!tmpfile) goto scan_exec_failed;
       fclose(*fileP);
       *fileP = tmpfile;
-      retval = psscan(fileP,filename_dsc,filename_raw,filename_dscP,cmd_scan_pdf,NULL,NULL,scanstyle);
+      retval = psscan(fileP,filename_dsc,filename_raw,filename_dscP,cmd_scan_pdf,NULL,NULL,scanstyle,gv_gs_safeDir);
       if (!retval) {
 	sprintf(s,"Scanning\n%s\nfailed.",filename_dsc);
 	goto scan_failed;
@@ -1738,6 +1764,7 @@ static char * ps_io_fgetchars(fd,num)
             INFMESSAGE(buffer became to large)
             ENDMESSAGE(ps_io_fgetchars)
 	    fprintf(stderr, "gv: ps_io_fgetchars: Fatal Error: buffer became too large.\n");
+	    clean_safe_tempdir();
 	    exit(-1);
          }
          if (FD_LINE_BEGIN) {
