@@ -228,7 +228,7 @@ resource_buildDatabase (
   }
 
   s = resource_getResource(db,app_class,app_name, "international",0);
-  if (!strcasecmp(s, "False"))
+  if (s == NULL || !strcasecmp(s, "False"))
   {
      sprintf(locale1, "noint:%s%s", loc_lang, loc_terr);
      sprintf(locale2, "noint:%s", loc_lang);
@@ -440,6 +440,14 @@ resource_buildDatabase (
     {
       resource_putResource (&db, app_name, ".gsSafer", "False");
     }
+  if (safedir_p)
+    {
+      resource_putResource (&db, app_name, ".gsSafeDir", "True");
+    }
+  if (nosafedir_p)
+    {
+      resource_putResource (&db, app_name, ".gsSafeDir", "False");
+    }
   if (pixmap_p)
     {
       resource_putResource (&db, app_name, ".useBackingPixmap", "True");
