@@ -40,8 +40,8 @@
 #define  VERSION_APPLIC_CONTEXT	app_con  			/* the Application Context */
 
 char *versionIdentification[] = {
-        "gv 3.6.90",
-        "April 2010",
+        "gv 3.6.91",
+        "May 2010",
 	0
 };
 
